package com.alienfactory.javamappy;



// -----------------------------------------------------------------------------
/**
 * A <code>JavaMappyException</code> is thrown when something fishy happens!
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class JavaMappyException extends RuntimeException {

	private Exception nestedException;



	// -------------------------------------------------------------------------
	// Constructors
	// -------------------------------------------------------------------------

	/**
	 * Constructs a <code>MapException</code> with the given message. <br>
	 * <br>
	 * @param strErrorMessage	an explanation this <code>Exception</code> was thrown
	 **/
	public JavaMappyException(String strErrorMessage) {
		super(strErrorMessage);
	}



	/**
	 * Constructs a <code>MapException</code> with a nested exception and an
	 * error message. <br>
	 * <br>
	 * @param strErrorMessage	the <code>MapException</code> message
	 * @param nestedException	the nested exception
	 **/
	public JavaMappyException(String strErrorMessage, Exception nestedException) {
		this(strErrorMessage);
		this.nestedException = nestedException;
	}



	// -------------------------------------------------------------------------
	// Public Methods
	// -------------------------------------------------------------------------

	/**
	 * Returns the nested exception if one exists. <code>null</code> is returned
	 * otherwise. <br>
	 * <br>
	 * @return	the nested exception, or <code>null</code>.
	 **/
	public Exception getNestedException() {
		return nestedException;
	}



	/**
	 * Returns the error message. If this exception contains a nested exception,
	 * then its message is appended to the end. <br>
	 * <br>
	 * @return	the error message
	 **/
	public String getMessage() {

		// grab our message
		String message = super.getMessage();

		// if a nested exception exists, then append its message to our own.
		if (nestedException != null) {
			message += "\nNested Exception :: " + nestedException.getClass().getName() + " - " + nestedException.getMessage();
		}

		// return the exception message(s).
		return message;
	}
}
