package com.alienfactory.javamappy;



//------------------------------------------------------------------------------
/**
 * A standard bean class representing a .FMP map. See
 * {@link com.alienfactory.javamappy.viewer.MapViewer} for details on rendering
 * a <code>Map</code> to the screen. <br>
 *
 * @author  Steve Eynon
 **/
//------------------------------------------------------------------------------
public class Map {

	/**
	 * This is the maximum number of {@link Layer Layers} that can be contained
	 * within a <code>Map</code>. <br>
	 * Has a constant value of <code>8</code>.
	 **/
	public static final int MAX_NO_OF_LAYERS	= 8;


	private MapHeader 	mapHeader		= new MapHeader();
	private Layer		layers[]		= new Layer[0];
	private Block		blocks[]		= new Block[0];
	private MapObject[] mapObjects 		= new MapObject[0];
	private String[] 	mapObjectImages = new String[0];
	private AnimBlock	animBlocks[]	= new AnimBlock[0];
	private int[]		colours			= new int[0];
	private PixelData 	imageData[]		= new PixelData[0];
	private String[]	description;
	private String[]	textStrings;



	//--------------------------------------------------------------------------
	// Public Getters and Setters
	//--------------------------------------------------------------------------

	public MapHeader getMapHeader() {
		return mapHeader;
	}

	public Layer[] getLayers() {
		return layers;
	}

	public Block[] getBlocks() {
		return blocks;
	}

	public MapObject[] getMapObjects() {
		return mapObjects;
	}

	public String[] getMapObjectImages() {
		return mapObjectImages;
	}

	public AnimBlock[] getAnimBlocks() {
		return animBlocks;
	}

	public int[] getColours() {
		return colours;
	}

	public PixelData[] getImageData() {
		return imageData;
	}

	public void setDescription(String[] description) {
		this.description = description;
	}

	public String[] getDescription() {
		return description;
	}

	public String[] getTextStrings() {
		return textStrings;
	}

	public void setLayers(Layer[] layers) {
		this.layers = layers;
	}

	/**
	 * <code>nulls</code> are allowed. <br>
	 */
	public void setAnimBlocks(AnimBlock[] animBlocks) {
		this.animBlocks = animBlocks;
	}

	/**
	 * <code>nulls</code> are allowed. <br>
	 */
	public void setBlocks(Block[] blocks) {
		this.blocks = blocks;
	}

	/**
	 * <code>nulls</code> are allowed. <br>
	 */
	public void setMapObjects(MapObject[] mapObjects) {
		this.mapObjects = mapObjects;
	}

	/**
	 * <code>nulls</code> are allowed. <br>
	 */
	public void setMapObjectImages(String[] mapObjectImages) {
		this.mapObjectImages = mapObjectImages;
	}

	/**
	 * <code>nulls</code> are allowed. In fact, <code>pixelData</code> is not
	 * used by JavaMappy after the <code>Map</code> has loaded. <br>
	 */
	public void setImageData(PixelData[] imageData) {
		this.imageData = imageData;
	}

	/**
	 * <code>nulls</code> are allowed. In fact, <code>colours</code> are not
	 * used by JavaMappy after the <code>Map</code> has loaded. <br>
	 */
	public void setColours(int[] colours) {
		this.colours = colours;
	}

	public void setTextStrings(String[] textStrings) {
		this.textStrings = textStrings;
	}
}
