package com.alienfactory.javamappy;

import com.alienfactory.javamappy.util.ParameterChecker;



//------------------------------------------------------------------------------
/**
 * A simple bean class which represents a <code>MapObject</code> as used by
 * <code>Mappy</code>. <br>
 * <br>
 * The Object structure for the MapWin32 Pro editor version.<br>
 * <br>
 * A <code>MapObject</code> may contain multiple images or a tile graphic,
 * 7 fields of User data and 8 other flags. <br>
 *
 * @author  Kris Arndt
 **/
//------------------------------------------------------------------------------
public class MapObject {

	/**
	 * A <code>MapObject</code> image layer used to get / set its image index. <br>
	 * <br>
	 * Has a constant value of <code>0</code>.
	 * @see		#setImageIndex(int, int)
	 * @see		#getImageIndex(int)
	 **/
	public static final int	BACKGROUND		= 0;

	/**
	 * A <code>MapObject</code> image layer used to get / set its image index. <br>
	 * <br>
	 * Has a constant value of <code>1</code>.
	 * @see		#setImageIndex(int, int)
	 * @see		#getImageIndex(int)
	 **/
	public static final int	FOREGROUND1		= 1;

	/**
	 * A <code>MapObject</code> image layer used to get / set its image index. <br>
	 * <br>
	 * Has a constant value of <code>2</code>.
	 * @see		#setImageIndex(int, int)
	 * @see		#getImageIndex(int)
	 **/
	public static final int	FOREGROUND2		= 2;

	/**
	 * A <code>MapObject</code> image layer used to get / set its image index. <br>
	 * <br>
	 * Has a constant value of <code>3</code>.
	 * @see		#setImageIndex(int, int)
	 * @see		#getImageIndex(int)
	 **/
	public static final int	FOREGROUND3		= 3;

	/**
	 * A minimum index for User Data. <br>
	 * <br>
	 * Has a constant value of <code>1</code>.
	 * @see		#setUserData(int, int)
	 * @see		#getUserData(int)
	 **/
	public static final int	MIN_USER_DATA	= 1;

	/**
	 * A minimum index for User Data. <br>
	 * <br>
	 * Has a constant value of <code>7</code>.
	 * @see		#setUserData(int, int)
	 * @see		#getUserData(int)
	 **/
	public static final int	MAX_USER_DATA	= 7;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>0</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int	FLAG1 			= 0;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>1</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int	FLAG2 			= 1;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>2</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int	FLAG3 			= 2;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>3</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int	FLAG4 			= 3;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>4</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int	FLAG5 			= 4;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>5</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int	FLAG6 			= 5;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>6</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int	FLAG7 			= 6;

	/**
	 * A User defined flag. <br>
	 * <br>
	 * Has a constant value of <code>7</code>.
	 * @see		#setFlag(int, boolean)
	 * @see		#getFlag(int)
	 **/
	public static final int	FLAG8 			= 7;


	private	int		x, y;
	private int 	gfxId;
	private int 	tileId;		// Tile block id, if any (0 if none).
	private int 	gXOffset;
	private int 	gYOffset;
	private int 	gWidth;
	private int 	gHeight;
	private int 	gHandleXOff;
	private int 	gHandleYOff; /* handle pos, from gxoff, gyoff */
	private int 	show; /* handle display mode (TL, M, BM) */

	private int 	imageIndex[]		= new int[4];
	private int 	userData[]			= new int[7];
	private	boolean	flags[]				= new boolean[8];
	private	String	textString;



	//--------------------------------------------------------------------------
	// Public Methods
	//--------------------------------------------------------------------------

	public void setX(int x) {
		this.x = x;
	}

	public int getX() {
		return x;
	}

	public void setY(int y) {
		this.y = y;
	}

	public int getY() {
		return y;
	}

	public void setGfxId(int i) {
		gfxId = i;
	}

	public int getGfxId() {
		return gfxId;
	}

	public void setTileId(int i) {
		tileId = i;
	}

	public int getTileId() {
		return tileId;
	}

	public void setGXOffset(int i) {
		gXOffset = i;
	}

	public int getGXOffset() {
		return gXOffset;
	}

	public void setGYOffset(int i) {
		gYOffset = i;
	}

	public int getGYOffset() {
		return gYOffset;
	}

	public void setGWidth(int i) {
		gWidth = i;
	}

	public int getGWidth() {
		return gWidth;
	}

	public void setGHeight(int i) {
		gHeight = i;
	}

	public int getGHeight() {
		return gHeight;
	}

	public void setGHandleXOff(int i) {
		gHandleXOff = i;
	}

	public int getGHandleXOff() {
		return gHandleXOff;
	}

	public void setGHandleYOff(int i) {
		gHandleYOff = i;
	}

	public int getGHandleYOff() {
		return gHandleYOff;
	}

	public void setShow(int i) {
		show = i;
	}

	public int getShow() {
		return show;
	}

	/**
	 * Sets the image index of the given <code>MapObject</code> layer. <br>
	 * @param	imageLayer should be one of {@link #BACKGROUND},
	 * 			{@link #FOREGROUND1}, {@link #FOREGROUND2} or
	 * 			{@link #FOREGROUND3}
	 * @param	newImageIndex the image index to be set
	 * @throws	IllegalArgumentException if <code>imageLayer</code> is out of
	 * 			bounds - it is assumed that <code>newImageIndex</code> is valid
	 * @see		#getImageIndex(int)
	 **/
	public void setImageIndex(int imageLayer, int newImageIndex) throws IllegalArgumentException  {
		ParameterChecker.checkMinMax(imageLayer, "imageLayer", BACKGROUND, FOREGROUND3);
		imageIndex[imageLayer] = newImageIndex;
	}



	/**
	 * Returns the image index of the given <code>MapObject</code> layer. <br>
	 * @param	imageLayer should be one of {@link #BACKGROUND},
	 * 			{@link #FOREGROUND1}, {@link #FOREGROUND2} or
	 * 			{@link #FOREGROUND3}
	 * @return	the image index of the given <code>MapObject</code> layer.
	 * @throws	IllegalArgumentException if <code>imageLayer</code> is out of
	 * 			bounds
	 * @see		#setImageIndex(int, int)
	 **/
	public int getImageIndex(int imageLayer) throws IllegalArgumentException {
		if ((imageLayer < BACKGROUND) || (imageLayer > FOREGROUND3))
			 throw new IllegalArgumentException("Input parameter [imageLayer :: " + imageLayer + "] was out of bounds :: " + BACKGROUND + " >= imageLayer =< " + FOREGROUND3);

		// return image index
		return (imageIndex[imageLayer]);
	}




	/**
	 * Sets the specified User Data. <br>
	 * @param	index the User Data to be updated.
	 * @param	newUserData	the new User Data.
	 * @throws	IllegalArgumentException if
	 * 			(<code>index</code> &lt; {@link #MIN_USER_DATA}) or
	 * 			(<code>index</code> &gt; {@link #MAX_USER_DATA}) or
	 * @see		#getUserData(int)
	 **/
	public void setUserData(int index, int newUserData) throws IllegalArgumentException {
		ParameterChecker.checkMinMax(index, "index", MIN_USER_DATA, MAX_USER_DATA);
		userData[index - 1] = newUserData;
	}



	/**
	 * Returns the specified User Data. <br>
	 * @param	index the User Data to be returned.
	 * @return	the specified User Data.
	 * @throws	IllegalArgumentException if
	 * 			(<code>index</code> &lt; {@link #MIN_USER_DATA}) or
	 * 			(<code>index</code> &gt; {@link #MAX_USER_DATA}) or
	 * @see		#setUserData(int, int)
	 **/
	public int getUserData(int index) throws IllegalArgumentException {
		if ((index < MIN_USER_DATA) || (index > MAX_USER_DATA))
			 throw new IllegalArgumentException("Input parameter [index :: " + index + "] was out of bounds :: " + MIN_USER_DATA + " >= index =< " + MAX_USER_DATA);

		// return User Data
		return userData[index - 1];
	}



	/**
	 * Sets the specified flag to the given boolean value. <br>
	 * <br>
	 * @param	flag should be between {@link #FLAG1} and {@link #FLAG2}
	 * @param	newFlag	the new flag to set
	 * @throws	IllegalArgumentException if <code>flag</code> is out of bounds
	 * @see		#getFlag(int)
	 **/
	public void setFlag(int flag, boolean newFlag) throws IllegalArgumentException {
		ParameterChecker.checkMinMax(flag, "flag", FLAG1, FLAG8);
		flags[flag] = newFlag;
	}



	/**
	 * Returns the specified flag. <br>
	 * <br>
	 * @param	flag should be between {@link #FLAG1} and {@link #FLAG2}
	 * @return	the specified flag value
	 * @throws	IllegalArgumentException if <code>flag</code> is out of bounds
	 * @see		#setFlag(int, boolean)
	 **/
	public boolean getFlag(int flag) throws IllegalArgumentException {
		if ((flag < FLAG1) || (flag > FLAG8))
			throw new IllegalArgumentException("Input parameter [flag :: " + flag + "] was out of bounds :: " + FLAG1 + " >= flag =< " + FLAG8);

		// return Flag Data
		return flags[flag];
	}



	/**
	 * Returns the text string associated with this MapObject. The string is set
	 * when the map is loaded and is defined within Mappy. <br>
	 */
	public String getTextString() {
		return textString;
	}


	/**
	 * Sets the text string associated with this MapObject. <br>
	 */
	public void setTextString(String textString) {
		this.textString = textString;
	}
}
