package com.alienfactory.javamappy.loader;

import com.alienfactory.javamappy.AnimBlock;
import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.util.Logger;



// -----------------------------------------------------------------------------
/**
 * Chunk ANDT :: Contains ANISTR structures. <br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class ChunkLoaderANDT extends ChunkLoader {
	private final Logger log = Logger.createLogger(super.getClass());



	// -------------------------------------------------------------------------
	// Public Methods
	// -------------------------------------------------------------------------

	// JavaDoc inherited from ChunkLoader
	public void loadChunk(Map map) throws ChunkLoadException {
		int aniBlockCount = 0;
		int index = super.getChunkLength() - 16;
		while (super.getByte(index) != -1) {
			aniBlockCount++;
			index -= 16;
		}

		int intSize		= 1;
		int	imageSize	= 1;
		if (map.getMapHeader().getMapType() == 0) {
			imageSize 	= map.getMapHeader().getBlockSize();
			intSize		= 4;
		}

		log.info("Reading [" + aniBlockCount + "] AniBlock properties...");

		int offset1, offset2, noOfFrames, startFrame;
		int frames[];

		AnimBlock animBlocks[] = new AnimBlock[aniBlockCount];
		index = super.getChunkLength() - 16;
		aniBlockCount = 0;
		AnimBlock animBlock;
		StringBuffer sequence;
		while (super.getByte(index) != -1) {
			animBlock = new AnimBlock();

			animBlock.setType		(super.getByte(index+0));
			animBlock.setDelay		(super.getByte(index+1));
			animBlock.setDelayCountdown(super.getByte(index+2));
			animBlock.setUserData	(super.getByte(index+3));

			offset1 				= getInt(index + 8);
			offset2 				= getInt(index + 12);
			noOfFrames 				= (offset2 - offset1) / intSize;
			frames 					= new int[noOfFrames];
			startFrame 				= (getInt(index + 4) - offset1) / intSize;
			animBlock.setFrames		(frames);

			// we can't set the current frame if it hasn't got any frames!
			if (noOfFrames > 0)
				animBlock.setCurrentFrameIndex(startFrame);

			// convert values between map types
			if (map.getMapHeader().getMapType() != 0)
				offset1 = (offset1 * 4) - super.getChunkLength();

			sequence = new StringBuffer(noOfFrames * 4);
			for (int j=0; j<noOfFrames; j++) {
				frames[j]  = getInt(super.getChunkLength() + offset1 - 4);
				frames[j] /= imageSize;
				offset1 += 4;
				sequence.append(frames[j]);
				if (j != (noOfFrames-1))
					sequence.append(", ");
			}
			index -= 16;

			log.debug("AniBlock [" + aniBlockCount + "] sequence = [" + sequence + "]");

			animBlocks[aniBlockCount++] = animBlock;
		}

		map.setAnimBlocks(animBlocks);

		super.skip(super.getBytesRemaining());
	}



	// returns an int at the given index without affecting the inputStream
	private int getInt(int index) throws ChunkLoadException {
        int 	value = 0;
        byte[]	raw = new byte[4];

        raw[0] = super.getByte(index + 0);
        raw[1] = super.getByte(index + 1);
        raw[2] = super.getByte(index + 2);
        raw[3] = super.getByte(index + 3);

        // take into account the effects of LSB
        if (super.isLSB()) {
            byte temp;
            temp   = raw[0];
            raw[0] = raw[3];
            raw[3] = temp;
            temp   = raw[1];
            raw[1] = raw[2];
            raw[2] = temp;
        }

        // loop for all the required bytes
        for (int i=0; i<4; i++) {
            value <<= 8;
            value |= raw[i] & 0xFF;
        }

        // return int value
        return value;
	}

}
