package com.alienfactory.javamappy.loader;

import com.alienfactory.javamappy.Block;
import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.MapHeader;
import com.alienfactory.javamappy.util.Logger;



// -----------------------------------------------------------------------------
/**
 * Chunk BKDT :: Contains BLKSTR structures. <br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class ChunkLoaderBKDT extends ChunkLoader {
	private final Logger log = Logger.createLogger(super.getClass());



	// -------------------------------------------------------------------------
	// Public Methods
	// -------------------------------------------------------------------------

	// JavaDoc inherited from ChunkLoader
	public void loadChunk(Map map) throws ChunkLoadException {
		MapHeader 	mapHeader 		= map.getMapHeader();
		Block[]		blocks 			= new Block[mapHeader.getNoOfBlocks()];
		int			imageByteSize	= 1;
		int 		bits;
		int			textStringUdi	= mapHeader.getTextStringUserDataIndex();
		String[]	textStrings		= map.getTextStrings();
		int			textStringIndex;

		if (mapHeader.getMapType() == 0)
			imageByteSize = mapHeader.getBlockWidth() * mapHeader.getBlockHeight() * ((mapHeader.getColourDepth() + 1) / 8);	// the "+1" caters for a colour depth of 15

		log.info("Reading [" + mapHeader.getNoOfBlocks() + "] block properties...");
		for (int i = 0; i < mapHeader.getNoOfBlocks(); i++) {
			Block block = new Block();

			block.setImageIndex(Block.BACKGROUND,  super.readInt() / imageByteSize);
			block.setImageIndex(Block.FOREGROUND1, super.readInt() / imageByteSize);
			block.setImageIndex(Block.FOREGROUND2, super.readInt() / imageByteSize);
			block.setImageIndex(Block.FOREGROUND3, super.readInt() / imageByteSize);

			block.setUserData(1, super.readInt());
			block.setUserData(2, super.readInt());
			block.setUserData(3, super.readShort());
			block.setUserData(4, super.readShort());
			block.setUserData(5, super.readUByte());
			block.setUserData(6, super.readUByte());
			block.setUserData(7, super.readUByte());

			bits = super.readUByte();
			block.setCollisionFlag(Block.TOP | Block.LEFT,		(bits & 0x01) != 0);
			block.setCollisionFlag(Block.TOP | Block.RIGHT,		(bits & 0x02) != 0);
			block.setCollisionFlag(Block.BOTTOM | Block.LEFT,	(bits & 0x04) != 0);
			block.setCollisionFlag(Block.BOTTOM | Block.RIGHT,	(bits & 0x08) != 0);
			block.setFlag(Block.TRIGGER,						(bits & 0x10) != 0);
			block.setFlag(Block.UNUSED1,						(bits & 0x20) != 0);
			block.setFlag(Block.UNUSED2,						(bits & 0x40) != 0);
			block.setFlag(Block.UNUSED3,						(bits & 0x80) != 0);

			if (textStringUdi > -1) {
				textStringIndex = block.getUserData(textStringUdi);
				block.setTextString(textStrings[textStringIndex]);
			}

			blocks[i] = block;
		}

		map.setBlocks(blocks);
	}
}
