package com.alienfactory.javamappy.loader;

import com.alienfactory.javamappy.Layer;
import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.MapHeader;



// -----------------------------------------------------------------------------
/**
 * Chunk BODY :: An array of short ints containing positive offsets into BKDT,
 * and negative offsets into ANDT. I.e. this is the actual map data. <br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class ChunkLoaderBODY extends ChunkLoader {



	// -------------------------------------------------------------------------
	// Public Methods
	// -------------------------------------------------------------------------

	public void loadChunk(Map map) throws ChunkLoadException {
		MapHeader mapHeader = map.getMapHeader();

		short	mapLayer[][] = new short[mapHeader.getMapBlockHeight()][mapHeader.getMapBlockWidth()];

		for (int y=0; y < mapHeader.getMapBlockHeight(); y++) {
			for (int x=0; x < mapHeader.getMapBlockWidth(); x++) {

				int data = super.readShort();

				// does the RLE not wrap around rows?
				if (mapHeader.getMapType() == 2) {	//map is RLE compressed
					int rleCount = data;
					if (rleCount >= 32768) rleCount -= 65536;

					if (rleCount > 0) {
						while (rleCount > 0) {
							data = super.readShort();
							mapLayer[y][x] = (short) data;
							x++; rleCount--;
						}
					} else {
						data = super.readShort();
						short rleval = (short) data;
						while (rleCount < 0) {
							mapLayer[y][x] = rleval;
							x++; rleCount++;
						}
					}
					x--; //correct for RLE
				} else {
					mapLayer[y][x] = (short) data;

					if (mapLayer[y][x] >= 0) {
						if (mapHeader.getMapType() == 0) mapLayer[y][x] /= mapHeader.getBlockSize();
					} else {
						if (mapHeader.getMapType() == 0) mapLayer[y][x] /= 16;
					}
				}
			}
		}

		// find the chunk layer index
        int layerIndex = 0;
        if (!super.getChunkName().equals("BODY")) {
	        String strLayerIndex = super.getChunkName().substring(3);
	        try {
	        	layerIndex = Integer.parseInt(strLayerIndex);
	        } catch (NumberFormatException nfe) {
	        	super.throwChunkException("Cannot find Layer index, invalid chunk name :: chunkName=[" + super.getChunkName() + "]");
	        }
        }

		Layer layer = new Layer(map.getMapHeader(), mapLayer, map.getBlocks(), map.getAnimBlocks());

		// make sure the layers array is large enough to accommodate the new layer
		if ((layerIndex + 1) > map.getLayers().length) {
			Layer[] layers = new Layer[layerIndex + 1];
			System.arraycopy(map.getLayers(), 0, layers, 0, map.getLayers().length);
			map.setLayers(layers);
		}

		map.getLayers()[layerIndex] = layer;
	}

}
