package com.alienfactory.javamappy.loader;

import com.alienfactory.javamappy.MapObject;
import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.MapHeader;
import com.alienfactory.javamappy.util.Logger;



//------------------------------------------------------------------------------
/**
 * Chunk OBDT :: Contains OBJSTR structures. <br>
 * Load the Objects from the MapWin32 Pro editor.<br>
 *
 * @author Kris Arndt
 **/
// ------------------------------------------------------------------------------
public class ChunkLoaderOBDT extends ChunkLoader {
	private final Logger log = Logger.createLogger(super.getClass());

	// --------------------------------------------------------------------------
	// Public Methods
	// --------------------------------------------------------------------------

	// JavaDoc inherited from ChunkLoader
	public void loadChunk(Map map) throws ChunkLoadException {
		MapHeader mapHeader = map.getMapHeader();
		int bits;
		int textStringUdi = mapHeader.getTextStringUserDataIndex();
		String[] textStrings = map.getTextStrings();
		int textStringIndex;

		int i = super.readInt(); /* Offset to OBJSTR */
		if (i < 16)
			throw new ChunkLoadException("Offset to objstr was less than 16.");
		int numObjects = super.readInt();
		super.readInt();	// curobj
		int objectStructureSize = super.readInt();

		super.skip(16 - i);

		MapObject[] mapObjects = new MapObject[numObjects];

		log.info("Reading [" + numObjects + "] Object properties...");

		for (i = 0; i < numObjects; i++) {
			MapObject mo = new MapObject();

			int bytesRemaining = super.getBytesRemaining();

			mo.setX				(super.readInt());
			mo.setY				(super.readInt());
			mo.setGfxId			(super.readInt());
			mo.setTileId		(super.readInt());
			mo.setGXOffset		(super.readInt());
			mo.setGYOffset		(super.readInt());
			mo.setGWidth		(super.readInt());
			mo.setGHeight		(super.readInt());
			mo.setGHandleXOff	(super.readInt());
			mo.setGHandleYOff	(super.readInt());
			mo.setShow			(super.readInt());

			for (int j = 1; j < 8; j++)
				mo.setUserData(j, super.readInt());

			bits = super.readInt();

			mo.setFlag(MapObject.FLAG1, (bits & 0x01) != 0);
			mo.setFlag(MapObject.FLAG2, (bits & 0x02) != 0);
			mo.setFlag(MapObject.FLAG3, (bits & 0x04) != 0);
			mo.setFlag(MapObject.FLAG4, (bits & 0x08) != 0);
			mo.setFlag(MapObject.FLAG5, (bits & 0x10) != 0);
			mo.setFlag(MapObject.FLAG6, (bits & 0x20) != 0);
			mo.setFlag(MapObject.FLAG7, (bits & 0x40) != 0);
			mo.setFlag(MapObject.FLAG8, (bits & 0x80) != 0);

			if (textStringUdi > -1) {
				textStringIndex = mo.getUserData(textStringUdi);
				if ((textStringIndex >= 0)
						&& (textStringIndex < textStrings.length))
					mo.setTextString(textStrings[textStringIndex]);
			}

			mapObjects[i] = mo;

			super.skip(objectStructureSize - (bytesRemaining - super.getBytesRemaining()));
		}

		map.setMapObjects(mapObjects);
	}
}
