package com.alienfactory.javamappy.loader;

import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.MapHeader;
import com.alienfactory.javamappy.util.Logger;



// -----------------------------------------------------------------------------
/**
 * Chunk TSTR :: Text Strings. <br>
 *
 * @author Steve Eynon
 */
// -----------------------------------------------------------------------------
public class ChunkLoaderTSTR extends ChunkLoader {
	private final Logger log = Logger.createLogger(super.getClass());



	// -------------------------------------------------------------------------
	// Public Methods
	// -------------------------------------------------------------------------

	public void loadChunk(Map map) throws ChunkLoadException {
		MapHeader 	mapHeader 	= map.getMapHeader();

		int bytesToStart 		= super.readInt();
		int noOfStrings			= super.readInt();
		int userDataIndex		= super.readInt();
		String[] textStrings	= new String[noOfStrings];

		// consume extra data
		bytesToStart -= (3 * 4);
		super.skip(bytesToStart);

		// read the strings
		if (Logger.isLoggingEnabled())
			log.info("Reading [" + noOfStrings + "] Text Strings...");

		for (int i = 0; i< noOfStrings; i++) {
			textStrings[i] = super.readString();
			log.debug("TextString [" + i + "] :: " + textStrings[i]);
		}

		// save our data for others
		mapHeader.setTextStringUserDataIndex(userDataIndex);
		map.setTextStrings(textStrings);

		// there might be the odd byte left over to keep the addressing even
		super.skip(super.getBytesRemaining());
	}

}
