package com.alienfactory.javamappy.util;



// -----------------------------------------------------------------------------
/**
 * This simple Logger logs messages to <code>System.out</code>. <br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class Logger {

	private static boolean loggingEnabled;
	private String className;



	// -------------------------------------------------------------------------
	// Constructors
	// -------------------------------------------------------------------------

	private Logger(Class clazz) {
		className = clazz.getName();

		// strip off the package names - I can guess what they are!
		int intDot = className.lastIndexOf('.');
		if (intDot > 0)
			className = className.substring(intDot + 1);
	}

	private Logger(String className) {
		this.className = className;
	}



	// -------------------------------------------------------------------------
	// Factory Methods
	// -------------------------------------------------------------------------

	public static Logger createLogger(Class clazz) {
		if (clazz == null) throw new IllegalArgumentException("Input parameter [clazz] was null.");
		return new Logger(clazz);
	}

	public static Logger createLogger(String className) {
		if (className == null) throw new IllegalArgumentException("Input parameter [className] was null.");
		return new Logger(className);
	}

	public static boolean isLoggingEnabled() {
		return loggingEnabled;
	}

	public static void setLoggingEnabled(boolean b) {
		loggingEnabled = b;
	}



	// -------------------------------------------------------------------------
	// Logging Methods
	// -------------------------------------------------------------------------

	public void trace(Object obj) {
		//	log("TRACE", obj.toString());
	}

	public void debug(Object obj) {
		log("DEBUG", obj.toString());
	}

	public void info(Object obj) {
		log("INFO", obj.toString());
	}

	public void warn(Object obj) {
		log("WARN", obj.toString());
	}

	public void error(Object obj, Exception e) {
		log("ERROR", obj.toString());
		error(e);
	}

	public void error(Exception e) {
		// errors should never die silently
		log("ERROR", e.getMessage());
		e.printStackTrace();
	}



	// -------------------------------------------------------------------------
	// Private Methods
	// -------------------------------------------------------------------------

	private void log(String level, String message) {
		if (Logger.isLoggingEnabled()) {
			StringBuffer buff = new StringBuffer();

			buff.append("<");
			buff.append(level);
			buff.append(">");

			while (buff.length() < 7)
				buff.append(" ");

			buff.append(" ");

			buff.append("<");
			buff.append(className);
			buff.append(">");

			buff.append(" ");

			buff.append("<");
			buff.append(message);
			buff.append(">");

			System.out.println(buff.toString());
		}
	}
}
