package com.alienfactory.javamappy.viewer.render;

import java.awt.Image;
import java.awt.image.BufferedImage;

import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.PixelData;



// -----------------------------------------------------------------------------
/**
 * This <code>Renderer</code> has been optimised for use in JDK 1.2 JVMs. <br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
public class JDK12Renderer extends JDKRenderer {


	/**
	 * Uses the raw pixel data and colour map from the given <code>Map</code> to
	 * create the solid and transparent images in a JDK specific manner. <br>
	 * <br>
	 * @param map the {@link Map} containing the image data to be rendered
	 * @throws IllegalArgumentException if <code>Map</code> is <code>null</code>
	 **/
	public JDK12Renderer(Map map) throws IllegalArgumentException {
		super(map);

		PixelData imageData[]	= map.getImageData();
		super.opaqueImages		= new Image[imageData.length];
		super.transparentImages = new Image[imageData.length];

		BufferedImage buffImage;
		for (int i=0; i < imageData.length; i++) {
			buffImage = new BufferedImage(blockWidth, blockHeight, BufferedImage.TYPE_INT_RGB);
			buffImage.setRGB(0, 0, blockWidth, blockHeight, imageData[i].getRawPixelData(), 0, blockWidth);
			super.opaqueImages[i] = buffImage;

			// don't bother making a a transparent image if it has no transparent pixels
			if (imageData[i].hasTransparentPixels()) {
				buffImage = new BufferedImage(blockWidth, blockHeight, BufferedImage.TYPE_INT_ARGB);
				buffImage.setRGB(0, 0, blockWidth, blockHeight, imageData[i].getRawPixelData(), 0, blockWidth);
				super.transparentImages[i] = buffImage;
			}
		}
	}

}
