package com.alienfactory.javamappy.viewer.render;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.Shape;

import com.alienfactory.javamappy.Map;
import com.alienfactory.javamappy.util.ParameterChecker;



// -----------------------------------------------------------------------------
/**
 * JDKRenderer <br>
 *
 * @author Steve Eynon
 **/
// -----------------------------------------------------------------------------
abstract class JDKRenderer implements Renderer {
	private 	Shape 		oldClipShape;
	protected 	Image[]		opaqueImages;
	protected 	Image[]		transparentImages;
	protected	final int	blockWidth;
	protected	final int	blockHeight;
	private 	final int	halfBlockWidth;



	// -------------------------------------------------------------------------
	// Constructors
	// -------------------------------------------------------------------------

	JDKRenderer(Map map) {
		ParameterChecker.checkNotNull(map, "map");
		blockWidth  	= map.getMapHeader().getBlockWidth();
		blockHeight 	= map.getMapHeader().getBlockHeight();
		halfBlockWidth	= blockWidth / 2;
	}



	// -------------------------------------------------------------------------
	// Renderer Interface Methods
	// -------------------------------------------------------------------------

	public void setClip(Object objGFX, int x, int y, int width, int height) throws IllegalArgumentException {
		Graphics gfx = (Graphics) objGFX;

		// let the system do the clipping for us
		oldClipShape = gfx.getClip();
		gfx.setClip(x, y, width, height);
	}



	public void restoreClip(Object objGFX) throws IllegalArgumentException {
		Graphics gfx = (Graphics) objGFX;

		gfx.setClip(oldClipShape);
		oldClipShape = null;
	}



	public final void drawImage(Object objGFX, int imageIndex, int dstPixX, int dstPixY, boolean transparency, int modification) throws IllegalArgumentException {
		Graphics 	gfx 	= (Graphics) objGFX;
		Image		image	= null;

		// grab the appropriate image
		if (transparency) {
			image = transparentImages[imageIndex];
			if (image == null)
				image = opaqueImages[imageIndex];
		} else
			image = opaqueImages[imageIndex];

		// and draw it
		switch (modification) {
		case Renderer.NONE:
			gfx.drawImage(image, dstPixX, dstPixY, null);
			break;

		case Renderer.DRAW_LEFT_SIDE_ONLY:
			gfx.drawImage(image, dstPixX, dstPixY, dstPixX+halfBlockWidth, dstPixY+blockHeight, 0, 0, halfBlockWidth, blockHeight, null);
			break;

		case Renderer.DRAW_RIGHT_SIDE_ONLY:
			gfx.drawImage(image, dstPixX+halfBlockWidth, dstPixY, dstPixX+blockWidth, dstPixY+blockHeight, halfBlockWidth, 0, blockWidth, blockHeight, null);
			break;
		}
	}

}
